// distribution boxbackup-0.10 (svn version: 494)
//  
// Copyright (c) 2003 - 2006
//      Ben Summers and contributors.  All rights reserved.
//  
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. All use of this software and associated advertising materials must 
//    display the following acknowledgment:
//        This product includes software developed by Ben Summers.
// 4. The names of the Authors may not be used to endorse or promote
//    products derived from this software without specific prior written
//    permission.
// 
// [Where legally impermissible the Authors do not disclaim liability for 
// direct physical injury or death caused solely by defects in the software 
// unless it is modified by a third party.]
// 
// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//  
//  
//  
// --------------------------------------------------------------------------
//
// File
//		Name:    CompressStream.h
//		Purpose: Compressing stream
//		Created: 27/5/04
//
// --------------------------------------------------------------------------

#ifndef COMPRESSSTREAM__H
#define COMPRESSSTREAM__H

#include "IOStream.h"

// --------------------------------------------------------------------------
//
// Class
//		Name:    CompressStream
//		Purpose: Compressing stream
//		Created: 27/5/04
//
// --------------------------------------------------------------------------
class CompressStream : public IOStream
{
public:
	CompressStream(IOStream *pStream, bool TakeOwnership,
		bool DecompressRead, bool CompressWrite, bool PassThroughWhenNotCompressed = false);
	~CompressStream();
private:
	// No copying (have implementations which exception)
	CompressStream(const CompressStream &);
	CompressStream &operator=(const CompressStream &);
public:

	virtual int Read(void *pBuffer, int NBytes, int Timeout = IOStream::TimeOutInfinite);
	virtual void Write(const void *pBuffer, int NBytes);
	virtual void WriteAllBuffered();
	virtual void Close();
	virtual bool StreamDataLeft();
	virtual bool StreamClosed();

protected:
	void CheckRead();
	void CheckWrite();
	void CheckBuffer();
	void WriteCompressedData(bool SyncFlush = false);

private:
	IOStream *mpStream;
	bool mHaveOwnership;
	bool mDecompressRead;
	bool mCompressWrite;
	bool mPassThroughWhenNotCompressed;
	// Avoid having to include Compress.h
	void *mpReadCompressor;
	void *mpWriteCompressor;
	void *mpBuffer;
	bool mIsClosed;
};

#endif // COMPRESSSTREAM__H

